unit MainFrm;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Menus, Stdctrls;

{$R SPRITES.RES } //      .

type

  TSprite = class
  private
    FWidth: integer;
    FHeight: integer;
    FLeft: integer;
    FTop: integer;
    FAndImage, FOrImage: TBitMap;
  public
    property Top: Integer read FTop write FTop;
    property Left: Integer read FLeft write FLeft;
    property Width: Integer read FWidth write FWidth;
    property Height: Integer read FHeight write FHeight;
    constructor Create;
    destructor Destroy; override;
  end;

  TMainForm = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    BackGnd1, BackGnd2: TBitMap;
    Sprite: TSprite;
    GoLeft,GoRight,GoUp,GoDown: boolean;
    procedure MyIdleEvent(Sender: TObject; var Done: Boolean);
    procedure DrawSprite;
  end;

const

  BackGround = 'BACK2.BMP';

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

constructor TSprite.Create;
begin
  inherited Create;
  {      ,  
        AND/OR (/)  
     }
  FAndImage := TBitMap.Create;
  FAndImage.LoadFromResourceName(hInstance, 'AND');

  FOrImage := TBitMap.Create;
  FOrImage.LoadFromResourceName(hInstance, 'OR');

  Left := 0;
  Top := 0;
  Height := FAndImage.Height;
  Width := FAndImage.Width;

end;

destructor TSprite.Destroy;
begin
  FAndImage.Free;
  FOrImage.Free;
  inherited Destroy;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  //    
  BackGnd1 := TBitMap.Create;
  with BackGnd1 do
  begin
    LoadFromResourceName(hInstance, 'BACK');
    Parent := nil;
    SetBounds(0, 0,  Width, Height);
  end;

  //    
  BackGnd2 := TBitMap.Create;
  BackGnd2.Assign(BackGnd1);


  //   
  Sprite := TSprite.Create;

  //   
  GoRight := true;
  GoDown := true;
  GoLeft := false;
  GoUp := false;

  {    OnIdle   MyIdleEvent,
         }
  Application.OnIdle := MyIdleEvent;
  //       
  ClientWidth := BackGnd1.Width;
  ClientHeight := BackGnd1.Height;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  //   ,     FormCreate()
  BackGnd1.Free;
  BackGnd2.Free;
  Sprite.Free;
end;

procedure TMainForm.MyIdleEvent(Sender: TObject; var Done: Boolean);
begin
  DrawSprite;
  {    OnIdle    
        }
  Done := False;
end;

procedure TMainForm.DrawSprite;
var
  OldBounds: TRect;
begin

  //      OldBounds
  with OldBounds do
  begin
    Left := Sprite.Left;
    Top := Sprite.Top;
    Right := Sprite.Width;
    Bottom := Sprite.Height;
  end;

  {    ,      ,
            }
  with Sprite do
  begin
    if GoLeft then
      if Left > 0 then
         Left := Left - 1
      else begin
         GoLeft := false;
         GoRight := true;
      end;

    if GoDown then
      if (Top + Height) < self.ClientHeight then
         Top := Top + 1
      else begin
        GoDown := false;
        GoUp := true;
      end;

    if GoUp then
      if  Top > 0 then
         Top := Top - 1
      else begin
        GoUp := false;
        GoDown := true;
      end;

    if GoRight then
    if (Left + Width) < self.ClientWidth then
      Left := Left + 1
    else begin
      GoRight := false;
      GoLeft := true;
    end;
  end;

  {       BackGnd2 
        BackGnd1 }
  with OldBounds do
    BitBlt(BackGnd2.Canvas.Handle, Left, Top, Right, Bottom,
           BackGnd1.Canvas.Handle, Left, Top, SrcCopy);

  {     "" ,  
       }
  with Sprite do
  begin
    {         
       ,    FAndImage  BackGnd2 }
    BitBlt(BackGnd2.Canvas.Handle, Left, Top, Width, Height,
           FAndImage.Canvas.Handle, 0, 0, SrcAnd);
    //       
    BitBlt(BackGnd2.Canvas.Handle, Left, Top, Width, Height,
           FOrImage.Canvas.Handle, 0, 0, SrcPaint);
  end;

  {         .  
     ,   ,  
      .      
       ,       
          BitBlt }
  with OldBounds do
     BitBlt(Canvas.Handle, Left - 2, Top - 2, Right + 2, Bottom + 2,
            BackGnd2.Canvas.Handle, Left - 2, Top - 2, SrcCopy);

end;

procedure TMainForm.FormPaint(Sender: TObject);
begin
  //      
  BitBlt(Canvas.Handle, 0, 0, ClientWidth, ClientHeight,
         BackGnd1.Canvas.Handle, 0, 0, SrcCopy);
end;

end.
